#=================================
#================================= 
#==                             ==
#==     Star1 Makefile.inc      ==
#==                             ==
#=================================
#=================================

#ABOUTME: This is the Makfile.inc used for our workstation, Star1.
#It assumes none of the extlib dependencies are installed.

#PLEASE SPECIFY static OR shared
L=shared

MACH=STAR

#Specify the number of jobs for make (make -j$N).
#Increasing the number of jobs decreases the time to compile.
N=8

##### Logical Groupings ##########

### NOTE ###
#If you have extlibs already installed, comment them out below,
#and change the someextlib_INC and someextlib_LIB paths. 
##########

#EXTLIBS
EXTLIBS=\
	mpich \
	trilinos \
	libconfig \
	R \
	perl
	
#FOR DEVS ONLY
SOURCES=\
	comm \
	exec

##### Setting absolute path to IEL_HOME ##########

### NOTE ###
#a relative path to IEL_HOME is defined in each individual makefile
##########

### NOTE ###
#the IEL_HOME variable can then be used to determine the absolute path
##########

### ***IMPORTANT*** ###
#the method (abspath) for determining the IEL_HOMEa may not work on all systems.
#if using abspath causes incorrect paths you must replace it by
#typing in the path to the top directory ex: IEL_HOMEa=/home/myusername/opendiel
#######################

IEL_HOMEa=$(abspath $(IEL_HOME))

###### Include Helper Makefile.inc Files ##########

##include $(IEL_HOMEa)/Makefile.module.inc

###### Top Level Directories ##########

LIB_DIR=$(IEL_HOMEa)/LIB
INCLUDE_DIR=$(IEL_HOMEa)/INC
BIN_DIR=$(IEL_HOMEa)/BIN
EXTLIB_DIR=$(IEL_HOMEa)/EXTLIB
MODULES_DIR=$(IEL_HOMEa)/APPLICATION
DRIVER_DIR=DRIVER
TOOLS_DIR=$(IEL_HOMEa)/TOOL
SOURCE_DIR=$(IEL_HOMEa)/SRC

##### SOURCES Paths ##########

#EXECUTIVE
EXEC_INC=-I$(INCLUDE_DIR)/$(MACH)
EXEC_LIB=-L$(LIB_DIR)/$(MACH) -lIELexec

#COMMUNICATION
COMM_INC=-I$(INCLUDE_DIR)/$(MACH)
COMM_LIB=-L$(LIB_DIR)/$(MACH) -lIEL

#MACHINE SPECIFIC
MACH_INC=-I$(INCLUDE_DIR)/$(MACH)
MACH_LIB=-L$(LIB_DIR)/$(MACH) -lIELexec -lIEL

##### EXTLIBS Paths ##########

#TRILINOS
TRILINOS_LIB=-L$(LIB_DIR)/trilinos -laztecoo -lepetra -ltriutils -lteuchoscomm -lteuchoscore -lteuchosparameterlist -lteuchosnumerics -lteuchosremainder
TRILINOS_INC=-I$(INCLUDE_DIR)/trilinos

#MPICH
MPICH_LIB=-L$(LIB_DIR)/mpich -lmpich
MPICH_INC=-I$(INCLUDE_DIR)/mpich

#LIBCONFIG
LIBCONFIG_LIB=-L$(LIB_DIR)/libconfig -lconfig -lconfig++
LIBCONFIG_INC=-I$(INCLUDE_DIR)/libconfig

#R
#R_LIB=-L$(LIB_DIR)/R -lR -lRblas -lRlapack
R_LIBS=/usr/lib64/R/lib
R_INC=-I$(INCLUDE_DIR)/R
R_LIB_DIR=$(EXTLIB_DIR)/R/R-shared-install/lib64/R/lib
R_HOME=/usr/lib64/R

#PERL
PERL_LIB=-L$(LIB_DIR)/perl -lperl
PERL_INC=-I$(INCLUDE_DIR)/perl

##### Compiler Setting ########## 

#COMPILERS
CC=gcc
CXX=g++
F77=f95
F90=f95

MPICC=mpicc
MPICXX=mpicxx
MPIF77=mpif77
MPIF90=mpif90

#MPICC=$(BIN_DIR)/mpich/mpicc
#MPICXX=$(BIN_DIR)/mpich/mpicxx
#MPIF77=$(BIN_DIR)/mpich/mpif77
#MPIF90=$(BIN_DIR)/mpich/mpif90

#### mpirun path #########
MPIRUN=mpirun
#MPIRUN=$(BIN_DIR)/mpich/mpirun
